﻿using System;
using System.Collections.Generic;
using System.Web;
using SubSonic;
using Newtonsoft.Json.Linq;

namespace AppBox
{
    /// <summary>
    /// 角色模块权限帮助类
    /// </summary>
    public static class RoleModulePowerHelper
    {
        private static Dictionary<int, Dictionary<string, RoleModulePower>> _roleModulePowerDic = new Dictionary<int, Dictionary<string, RoleModulePower>>();

        /// <summary>
        /// 更新缓存
        /// </summary>
        /// <param name="roleId"></param>
        public static void UpdateRoleModulePowerDic(int roleId)
        {
            if (_roleModulePowerDic.ContainsKey(roleId))
            {
                _roleModulePowerDic.Remove(roleId);
            }
        }

        /// <summary>
        /// 获取角色对应的全部模块权限列表
        /// </summary>
        /// <param name="roleId"></param>
        /// <returns></returns>
        public static Dictionary<string, RoleModulePower> GetRoleModulePowerDic(int roleId)
        {
            if (!_roleModulePowerDic.ContainsKey(roleId))
            {
                //SqlQuery q = new Select().From<XMenu>();
                //q.Where("1").IsEqualTo("1");

                //// 名称不是AppModule.None
                //q.And(XMenu.ModuleNameColumn).IsNotEqualTo(ModuleTypeHelper.Module2String(ModuleType.None));

                //// items 是全部的模块列表
                //XMenuCollection items = q.ExecuteAsCollection<XMenuCollection>();

                List<RoleModulePower> roleModules = RoleModulePowerHelper.GetRoleModulePowers(roleId, ModuleTypeHelper.GetAppModules());

                _roleModulePowerDic[roleId] = new Dictionary<string, RoleModulePower>();
                foreach (RoleModulePower roleModule in roleModules)
                {
                    _roleModulePowerDic[roleId].Add(roleModule.ModuleName, roleModule);
                }
            }

            return _roleModulePowerDic[roleId];

        }

        /// <summary>
        /// 获取角色可以显示的模块列表
        /// </summary>
        /// <param name="roleId"></param>
        /// <returns></returns>
        public static List<string> GetRoleCanReadModuleList(int roleId)
        {
            List<string> moduleNames = new List<string>();

            Dictionary<string, RoleModulePower> roleModulePowers = GetRoleModulePowerDic(roleId);

            foreach (string moduleName in roleModulePowers.Keys)
            {
                RoleModulePower roleModulePower = roleModulePowers[moduleName];

                if (roleModulePower.CanRead)
                {
                    moduleNames.Add(roleModulePower.ModuleName);
                }
            }

            return moduleNames;
        }

        /// <summary>
        /// 查询角色对模块列表（modules）中模块的管理权限
        /// </summary>
        /// <param name="roleId">角色ID</param>
        /// <param name="moduleNames">所有的模块列表</param>
        /// <returns></returns>
        public static List<RoleModulePower> GetRoleModulePowers(int roleId, string[] moduleNames)
        {
            List<RoleModulePower> myRoleModules = new List<RoleModulePower>();

            Dictionary<string, XRoleModule> roleModuleDic = GetRoleModuleDictionary(roleId);

            string noneModuleName = ModuleTypeHelper.Module2String(ModuleType.None);
            foreach (string moduleName in moduleNames)
            {
                if (moduleName == noneModuleName)
                {
                    continue;
                }

                RoleModulePower myRoleModule = new RoleModulePower();
                myRoleModule.Title = ModuleTypeHelper.GetTitle(moduleName);
                myRoleModule.ModuleName = moduleName;
                myRoleModule.RoleId = roleId;

                // 初始化模块的CanRead权限
                myRoleModule.CanRead = false;

                // 初始化模块的其他权限（JSON对象）
                myRoleModule.OtherPowers = new JObject();
                List<string> otherPowerNames = ModulePowerTypeHelper.GetModuleOtherPowerTypeDic(moduleName);
                foreach (string powerName in otherPowerNames)
                {
                    myRoleModule.OtherPowers.Add(powerName, JToken.FromObject(false));
                }


                // 如果用户对此角色进行了权限分配，则读取这些值并更新myRoleModule
                if (roleModuleDic.ContainsKey(moduleName))
                {
                    XRoleModule roleModule = roleModuleDic[moduleName];
                    myRoleModule.CanRead = roleModule.CanRead;

                    if (!String.IsNullOrEmpty(roleModule.Others) && roleModule.Others != "{}")
                    {
                        JObject dbOtherPowerObj = JObject.Parse(roleModule.Others);

                        IEnumerator<KeyValuePair<string, JToken>> enumerator = dbOtherPowerObj.GetEnumerator();
                        while (enumerator.MoveNext())
                        {
                            KeyValuePair<string, JToken> current = enumerator.Current;
                            myRoleModule.OtherPowers[current.Key] = current.Value;
                        }
                    }
                }


                myRoleModules.Add(myRoleModule);
            }

            return myRoleModules;
        }


        /// <summary>
        /// 从数据库查找角色对应的全部模块权限
        /// </summary>
        /// <param name="roleId"></param>
        /// <returns></returns>
        private static Dictionary<string, XRoleModule> GetRoleModuleDictionary(int roleId)
        {
            SqlQuery q = new Select().From<XRoleModule>();
            q.Where("1").IsEqualTo("1");

            q.And(XRoleModule.RoleIdColumn).IsEqualTo(roleId);

            XRoleModuleCollection roleModules = q.ExecuteAsCollection<XRoleModuleCollection>();


            Dictionary<string, XRoleModule> roleModuleDic = new Dictionary<string, XRoleModule>();
            foreach (XRoleModule roleModule in roleModules)
            {
                roleModuleDic.Add(roleModule.ModuleName, roleModule);
            }

            return roleModuleDic;
        }



    }
}
