﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using SubSonic;
using FineUI;

namespace AppBox.admin
{
    public partial class dept : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Module

        /// <summary>
        /// 当前页面所属的模块
        /// </summary>
        public override ModuleType Module
        {
            get
            {
                return ModuleType.CoreDept;
            }
        }

        #endregion

        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            // 权限检查
            CheckPowerNewWithButton(btnNew);


            btnNew.OnClientClick = Window1.GetShowReference("~/admin/dept_new.aspx", "新增部门");

            BindGrid();
        }

        private void BindGrid()
        {
            SqlQuery q = new Select().From<XDept>();


            q.OrderAsc(XDept.SortIndexColumn.ColumnName);
            XDeptCollection depts = q.ExecuteAsCollection<XDeptCollection>();

            List<CustomDept> mys = CustomDeptHelper.GetMyDeptCollection(depts);

            Grid1.DataSource = mys;
            Grid1.DataBind();
        }

        #endregion

        #region Events

        protected void Grid1_PreDataBound(object sender, EventArgs e)
        {
            // 数据绑定之前，进行权限检查
            CheckPowerEditWithWindowField(Grid1, "editField");
            CheckPowerDeleteWithLinkButtonField(Grid1, "deleteField");
        }

        protected void Grid1_RowCommand(object sender, GridCommandEventArgs e)
        {
            int id = Convert.ToInt32(Grid1.DataKeys[e.RowIndex][0]);

            if (e.CommandName == "Delete")
            {
                // 在操作之前进行权限检查
                if (!CheckPowerDelete())
                {
                    CheckPowerFailWithAlert();
                    return;
                }

                SqlQuery q = new Select().From<XDeptUser>();
                q.And(XDeptUser.DeptIdColumn).IsEqualTo(id);

                int userCountUnderThisDept = q.GetRecordCount();
                if (userCountUnderThisDept > 0)
                {
                    Alert.ShowInTop("删除失败！需要先清空属于此部门的用户！");
                    return;
                }


                XDept.Delete(id);

                BindGrid();
            }
        }

        protected void Window1_Close(object sender, EventArgs e)
        {
            BindGrid();
        }

        #endregion

    }
}
