﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using SubSonic;
using FineUI;

namespace AppBox.admin
{
    public partial class dept_user : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Module

        /// <summary>
        /// 当前页面所属的模块
        /// </summary>
        public override ModuleType Module
        {
            get
            {
                return ModuleType.CoreDeptUser;
            }
        }

        #endregion

        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            // 权限检查
            CheckPowerNewWithButton(btnNew);
            CheckPowerDeleteWithButton(btnDeleteSelected);


            ResolveDeleteButtonForGrid(btnDeleteSelected, Grid2, "确定要从当前部门中移除选中的{0}项记录吗？");


            BindGrid1();

            // 默认选中第一个部门
            //Grid1.SelectedRowIndexArray = new int[] { 0 };
            Grid1.SelectedRowIndex = 0;

            // 每页记录数
            Grid2.PageSize = XConfigHelper.PageSize;
            ddlGridPageSize.SelectedValue = XConfigHelper.PageSize.ToString();

            BindGrid2();
        }

        private void BindGrid1()
        {
            SqlQuery q = new Select().From<XDept>();


            q.OrderAsc(XDept.SortIndexColumn.ColumnName);
            XDeptCollection depts = q.ExecuteAsCollection<XDeptCollection>();

            List<CustomDept> mys = CustomDeptHelper.GetMyDeptCollection(depts);

            Grid1.DataSource = mys;
            Grid1.DataBind();
        }

        private void BindGrid2()
        {
            XUserCollection items;

            if (Grid1.SelectedRowIndexArray.Length != 0)
            {
                // 查询 X_User 表
                SqlQuery q = new Select().From<XUser>();
                q.Where("1").IsEqualTo("1");

                // 在用户名称中搜索
                string searchText = ttbSearchUser.Text.Trim();
                if (!String.IsNullOrEmpty(searchText))
                {
                    q.And(XUser.NameColumn).ContainsString(searchText);
                }

                // 过滤选中部门下的所有用户
                object[] values = Grid1.DataKeys[Grid1.SelectedRowIndexArray[0]];
                int roleId = Convert.ToInt32(values[0]);
                SqlQuery subQ = new Select(XDeptUser.UserIdColumn).From<XDeptUser>().Where(XDeptUser.DeptIdColumn).IsEqualTo(roleId);

                q.And(XUser.IdColumn).In(subQ);


                // 在查询添加之后，排序和分页之前获取总记录数
                // Grid1总共有多少条记录
                Grid2.RecordCount = q.GetRecordCount();

                // 排列
                q.OrderBys.Add(GetSortExpression(Grid2, XUser.Schema));

                // 数据库分页
                q.Paged(Grid2.PageIndex + 1, Grid2.PageSize);
                items = q.ExecuteAsCollection<XUserCollection>();

            }
            else
            {
                Grid2.RecordCount = 0;
                items = new XUserCollection();
            }

            Grid2.DataSource = items;
            Grid2.DataBind();
        }


        #endregion

        #region Events

        protected void ddlGridPageSize_SelectedIndexChanged(object sender, EventArgs e)
        {
            Grid2.PageSize = Convert.ToInt32(ddlGridPageSize.SelectedValue);

            BindGrid2();
        }


        #endregion

        #region Grid1 Events

        protected void Grid1_RowClick(object sender, FineUI.GridRowClickEventArgs e)
        {
            BindGrid2();
        }

        #endregion

        #region Grid2 Events

        protected void ttbSearchUser_Trigger2Click(object sender, EventArgs e)
        {
            ttbSearchUser.ShowTrigger1 = true;
            BindGrid2();
        }

        protected void ttbSearchUser_Trigger1Click(object sender, EventArgs e)
        {
            ttbSearchUser.Text = String.Empty;
            ttbSearchUser.ShowTrigger1 = false;
            BindGrid2();
        }

        protected void Grid2_PreDataBound(object sender, EventArgs e)
        {
            // 数据绑定之前，进行权限检查
            CheckPowerDeleteWithLinkButtonField(Grid2, "deleteField");
        }

        protected void Grid2_Sort(object sender, GridSortEventArgs e)
        {
            Grid2.SortColumnIndex = e.ColumnIndex;
            BindGrid2();
        }

        protected void Grid2_PageIndexChange(object sender, GridPageEventArgs e)
        {
            Grid2.PageIndex = e.NewPageIndex;
            BindGrid2();
        }

        protected void btnDeleteSelected_Click(object sender, EventArgs e)
        {
            // 在操作之前进行权限检查
            if (!CheckPowerDelete())
            {
                CheckPowerFailWithAlert();
                return;
            }

            // 从每个选中的行中获取ID（在Grid1中定义的DataKeyNames）
            List<int> roleIds = GetSelectedDataKeyIDs(Grid1);
            List<int> userIds = GetSelectedDataKeyIDs(Grid2);

            // 执行数据库操作
            //XUserCollection users = new Select().From<XUser>()
            //    .Where(XUser.IdColumn).In(ids)
            //    .ExecuteAsCollection<XUserCollection>();
            //foreach (XUser user in users)
            //{
            //    user.RoleId = null;
            //}
            //users.BatchSave();


            //XDeptUserCollection roleUsers = new Select().From<XDeptUser>()
            //    .Where(XDeptUser.DeptIdColumn).IsEqualTo(roleIds[0])
            //    .And(XDeptUser.IdColumn).In(userIds)
            //    .ExecuteAsCollection<XDeptUserCollection>();

            //foreach(XDeptUser roleUser in roleUsers)
            //{

            //}
            //roleUsers.BatchDelete();


            new Delete().From<XDeptUser>()
                .Where(XDeptUser.DeptIdColumn).IsEqualTo(roleIds[0])
                .And(XDeptUser.UserIdColumn).In(userIds)
                .Execute();


            // 清空当前选中的项
            Grid2.SelectedRowIndexArray = null;

            // 重新绑定表格
            BindGrid2();
        }


        protected void Grid2_RowCommand(object sender, GridCommandEventArgs e)
        {
            object[] values = Grid2.DataKeys[e.RowIndex];
            int userId = Convert.ToInt32(values[0]);

            if (e.CommandName == "Delete")
            {
                // 在操作之前进行权限检查
                if (!CheckPowerDelete())
                {
                    CheckPowerFailWithAlert();
                    return;
                }

                //XUser user = XUser.FetchByID(id);
                //user.RoleId = null;
                //user.Save();
                List<int> roleIds = GetSelectedDataKeyIDs(Grid1);

                new Delete().From<XDeptUser>()
                    .Where(XDeptUser.DeptIdColumn).IsEqualTo(roleIds[0])
                    .And(XDeptUser.UserIdColumn).IsEqualTo(userId)
                    .Execute();

                BindGrid2();

            }
        }

        protected void Window1_Close(object sender, EventArgs e)
        {
            BindGrid2();
        }

        protected void btnNew_Click(object sender, EventArgs e)
        {
            List<int> ids = GetSelectedDataKeyIDs(Grid1);
            string addUrl = String.Format("~/admin/dept_user_addnew.aspx?id={0}", ids[0]);

            PageContext.RegisterStartupScript(Window1.GetShowReference(addUrl, "添加用户到当前部门"));
        }

        #endregion

    }
}
