﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using SubSonic;
using FineUI;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;

namespace AppBox.admin
{
    public partial class dept_user_addnew : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Module

        /// <summary>
        /// 当前页面所属的模块
        /// </summary>
        public override ModuleType Module
        {
            get
            {
                return ModuleType.CoreDeptUser;
            }
        }

        #endregion

        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            // 权限检查
            if (!CheckPowerNew())
            {
                CheckPowerFailWithPage();
                return;
            }

            btnClose.OnClientClick = ActiveWindow.GetHideReference();

            int id = GetQueryIntValue("id");
            XDept current = XDept.FetchByID(id);
            if (current == null)
            {
                // 参数错误，首先弹出Alert对话框然后关闭弹出窗口
                Alert.Show("参数错误！", String.Empty, ActiveWindow.GetHideReference());
                return;
            }

            // 每页记录数
            Grid1.PageSize = XConfigHelper.PageSize;
            ddlGridPageSize.SelectedValue = XConfigHelper.PageSize.ToString();


            BindGrid();
        }


        private void BindGrid()
        {
            SqlQuery q = new Select().From<XUser>(); //.LeftOuterJoin(XDeptUser.UserIdColumn, XUser.IdColumn);
            q.Where("1").IsEqualTo("1");

            // 在职务名称中搜索
            string searchText = ttbSearchMessage.Text.Trim();
            if (!String.IsNullOrEmpty(searchText))
            {
                q.And(XUser.NameColumn).ContainsString(searchText);
            }

            // 排除所有已经属于某个部门的用户
            int currentDeptId = GetQueryIntValue("id");
            //SqlQuery subQ = new Select(XDeptUser.UserIdColumn).From<XDeptUser>().Where(XDeptUser.DeptIdColumn).IsEqualTo(currentDeptId);
            SqlQuery subQ = new Select(XDeptUser.UserIdColumn).From<XDeptUser>();

            q.And(XUser.IdColumn).NotIn(subQ);
            //q.And(XUser.IdColumn).IsNotEqualTo(1);

            // 只列出不在当前部门中的用户
            //q.AndExpression(XUser.DeptIdColumn.ColumnName).IsNotEqualTo(GetQueryIntValue("id")).Or(XUser.DeptIdColumn).IsNull().CloseExpression();

            // 在查询添加之后，排序和分页之前获取总记录数
            // Grid1总共有多少条记录
            Grid1.RecordCount = q.GetRecordCount();

            // 排列
            q.OrderBys.Add(GetSortExpression(Grid1, XUser.Schema));

            // 数据库分页
            q.Paged(Grid1.PageIndex + 1, Grid1.PageSize);
            XUserCollection items = q.ExecuteAsCollection<XUserCollection>();


            Grid1.DataSource = items;
            Grid1.DataBind();


            // 重新绑定表格数据之后，更新选中行
            UpdateSelectedRowIndexArray(hfSelectedIDS, Grid1);
        }

        #endregion

        #region Events

        protected void btnSaveClose_Click(object sender, EventArgs e)
        {
            SyncSelectedRowIndexArrayToHiddenField();

            int id = GetQueryIntValue("id");

            // 从每个选中的行中获取ID（在Grid1中定义的DataKeyNames）
            //List<int> ids = GetSelectedDataKeyIDs(Grid1);
            List<string> ids = GetSelectedIDsFromHiddenField(hfSelectedIDS);

            // 执行数据库操作
            //XUserCollection users = new Select().From<XUser>()
            //    .Where(XUser.IdColumn).In(ids)
            //    .ExecuteAsCollection<XUserCollection>();
            //foreach (XUser user in users)
            //{
            //    user.JobTitleId = roleId;
            //}
            //users.BatchSave();

            XDeptUserCollection deptUsers = new XDeptUserCollection();
            foreach (string userId in ids)
            {
                XDeptUser roleUser = new XDeptUser();
                roleUser.UserId = Convert.ToInt32(userId);
                roleUser.DeptId = id;

                deptUsers.Add(roleUser);
            }
            deptUsers.BatchSave();

            PageContext.RegisterStartupScript(ActiveWindow.GetHidePostBackReference());
        }

        private void SyncSelectedRowIndexArrayToHiddenField()
        {
            // 重新绑定表格数据之前，将当前表格页选中行的数据同步到隐藏字段中
            SyncSelectedRowIndexArrayToHiddenField(hfSelectedIDS, Grid1);
        }



        protected void ttbSearchMessage_Trigger2Click(object sender, EventArgs e)
        {
            SyncSelectedRowIndexArrayToHiddenField();

            ttbSearchMessage.ShowTrigger1 = true;
            BindGrid();
        }

        protected void ttbSearchMessage_Trigger1Click(object sender, EventArgs e)
        {
            SyncSelectedRowIndexArrayToHiddenField();

            ttbSearchMessage.Text = String.Empty;
            ttbSearchMessage.ShowTrigger1 = false;
            BindGrid();
        }

        protected void Grid1_Sort(object sender, GridSortEventArgs e)
        {
            SyncSelectedRowIndexArrayToHiddenField();

            Grid1.SortColumnIndex = e.ColumnIndex;
            BindGrid();
        }

        protected void Grid1_PageIndexChange(object sender, GridPageEventArgs e)
        {
            SyncSelectedRowIndexArrayToHiddenField();

            Grid1.PageIndex = e.NewPageIndex;
            BindGrid();
        }


        protected void ddlGridPageSize_SelectedIndexChanged(object sender, EventArgs e)
        {
            SyncSelectedRowIndexArrayToHiddenField();

            Grid1.PageSize = Convert.ToInt32(ddlGridPageSize.SelectedValue);

            BindGrid();
        }

        #endregion


    }
}
