﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using SubSonic;
using FineUI;


namespace AppBox.admin
{
    public partial class menu_new : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Module

        /// <summary>
        /// 当前页面所属的模块
        /// </summary>
        public override ModuleType Module
        {
            get
            {
                return ModuleType.CoreMenu;
            }
        }

        #endregion

        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            // 权限检查
            if (!CheckPowerNew())
            {
                CheckPowerFailWithPage();
                return;
            }

            btnClose.OnClientClick = ActiveWindow.GetHideReference();

            // 模块名称列表
            ddlModules.DataSource = ModuleTypeHelper.GetAppModules();
            ddlModules.DataBind();

            ddlModules.SelectedValue = ModuleTypeHelper.Module2String(ModuleType.None);

            BindDDL();

            menu_edit.InitIconList(iconList);
        }

        private void BindDDL()
        {
            List<CustomMenu> mys = ResolveDDL<CustomMenu>(CustomMenuHelper.GetCustomMenus());

            // 绑定到下拉列表（启用模拟树功能）
            ddlParent.EnableSimulateTree = true;
            ddlParent.DataTextField = "Title";
            ddlParent.DataValueField = "Id";
            ddlParent.DataSimulateTreeLevelField = "TreeLevel";
            ddlParent.DataSource = mys;
            ddlParent.DataBind();

            // 选中根节点
            ddlParent.SelectedValue = "0";
        }

        #endregion

        #region Events

        private void SaveItem()
        {
            XMenu item = new XMenu();
            item.ModuleName = ddlModules.SelectedValue;
            item.Title = tbxTitle.Text.Trim();
            item.NavigateUrl = tbxUrl.Text.Trim();
            item.SortIndex = Convert.ToInt32(tbxSortIndex.Text.Trim());
            //item.IsModule = cbxIsModule.Checked;
            item.ParentMenuId = Convert.ToInt32(ddlParent.SelectedValue);
            item.Remark = tbxRemark.Text.Trim();
            item.SyncMenuTitle = cbxSyncMenuTitle.Checked;
            item.Save();

            // 更新缓存
            CustomMenuHelper.UpdateCustomMenus();
        }

        protected void btnSaveClose_Click(object sender, EventArgs e)
        {
            SaveItem();

            //Alert.Show("添加成功！", String.Empty, ActiveWindow.GetHidePostBackReference());
            PageContext.RegisterStartupScript(ActiveWindow.GetHidePostBackReference());
        }

        //protected void btnSaveContinue_Click(object sender, EventArgs e)
        //{
        //    SaveMenu();

        //    BindDDL();

        //    FineUI.Alert.Show("添加成功！<br /><br />请继续添加新菜单。");
        //}

        protected void cbxSyncMenuTitle_CheckedChanged(object sender, EventArgs e)
        {
            UpdateMenuTitle();
        }

        protected void UpdateMenuTitle()
        {
            if (cbxSyncMenuTitle.Checked)
            {
                tbxTitle.Enabled = false;

                tbxTitle.Text = ModuleTypeHelper.GetTitle(ddlModules.SelectedValue);

            }
            else
            {
                tbxTitle.Enabled = true;
            }
        }

        #endregion

    }
}
