﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using SubSonic;
using FineUI;
using System.Transactions;
using System.Text;

namespace AppBox.admin
{
    public partial class user_edit : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Module

        /// <summary>
        /// 当前页面所属的模块
        /// </summary>
        public override ModuleType Module
        {
            get
            {
                return ModuleType.CoreUser;
            }
        }

        #endregion

        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            // 权限检查
            if (!CheckPowerEdit())
            {
                CheckPowerFailWithPage();
                return;
            }

            btnClose.OnClientClick = ActiveWindow.GetHideReference();

            int id = GetQueryIntValue("id");
            XUser current = XUser.FetchByID(id);
            if (current == null)
            {
                // 参数错误，首先弹出Alert对话框然后关闭弹出窗口
                Alert.Show("参数错误！", String.Empty, ActiveWindow.GetHideReference());
                return;
            }

            tbxName.Text = current.Name;
            tbxRealName.Text = current.ChineseName;
            tbxEmail.Text = current.CompanyEmail;
            tbxPersonalEmail.Text = current.PersonalEmail;
            tbxCellPhone.Text = current.CellPhone;
            tbxOfficePhone.Text = current.OfficePhone;
            tbxOfficePhoneExt.Text = current.OfficePhoneExt;
            tbxHomePhone.Text = current.HomePhone;
            tbxRemark.Text = current.Remark;
            cbxEnabled.Checked = current.Enabled;
            ddlGender.SelectedValue = current.Gender;

            //// 绑定部门下拉列表
            //BindDDLDept(current);


            // 初始化用户所属角色
            InitUserRole(current);

            // 初始化用户所属部门
            InitUserDept(current);

            // 初始化用户所属职称
            InitUserJobTitle(current);
        }

        #region InitUserRole

        private void InitUserDept(XUser current)
        {
            StringBuilder sbValue = new StringBuilder();
            StringBuilder sbText = new StringBuilder();

            // 初始化角色复选框列表的选择项
            XDeptCollection depts = new Select().From(XDept.Schema)
                .InnerJoin(XDeptUser.DeptIdColumn, XDept.IdColumn)
                .Where(XDeptUser.UserIdColumn).IsEqualTo(current.Id)
                .ExecuteAsCollection<XDeptCollection>();

            if (depts.Count > 0)
            {
                tbSelectedDept.Text = depts[0].Name;
                hfSelectedDept.Text = depts[0].Id.ToString();
            }

            // 打开编辑窗口
            string selectDeptURL = String.Format("./user_select_dept.aspx?ids=<script>{0}</script>", hfSelectedDept.GetValueReference());
            tbSelectedDept.OnClientTriggerClick = Window1.GetSaveStateReference(hfSelectedDept.ClientID, tbSelectedDept.ClientID)
                    + Window1.GetShowReference(selectDeptURL, "选择用户所属的部门");

        }

        #endregion

        #region InitUserRole

        private void InitUserRole(XUser current)
        {
            StringBuilder sbValue = new StringBuilder();
            StringBuilder sbText = new StringBuilder();

            // 初始化角色复选框列表的选择项
            XRoleCollection roles = new Select().From(XRole.Schema)
                .InnerJoin(XRoleUser.RoleIdColumn, XRole.IdColumn)
                .Where(XRoleUser.UserIdColumn).IsEqualTo(current.Id)
                .ExecuteAsCollection<XRoleCollection>();

            StringBuilder sb = new StringBuilder();
            foreach (XRole role in roles)
            {
                sbValue.AppendFormat("{0},", role.Id);
                sbText.AppendFormat("{0}，", role.Name);
            }

            string roleValues = sbValue.ToString().TrimEnd(',');
            string roleTexts = sbText.ToString().TrimEnd('，');

            tbSelectedRole.Text = roleTexts;
            hfSelectedRole.Text = roleValues;

            // 打开编辑角色的窗口
            string selectRoleURL = String.Format("./user_select_role.aspx?ids=<script>{0}</script>", hfSelectedRole.GetValueReference());
            tbSelectedRole.OnClientTriggerClick = Window1.GetSaveStateReference(hfSelectedRole.ClientID, tbSelectedRole.ClientID)
                    + Window1.GetShowReference(selectRoleURL, "选择用户所属的角色");

        } 
        #endregion

        #region InitUserJobTitle

        private void InitUserJobTitle(XUser current)
        {
            StringBuilder sbValue = new StringBuilder();
            StringBuilder sbText = new StringBuilder();

            // 初始化角色复选框列表的选择项
            XJobTitleCollection jobs = new Select().From(XJobTitle.Schema)
                .InnerJoin(XJobTitleUser.JobTitleIdColumn, XJobTitle.IdColumn)
                .Where(XJobTitleUser.UserIdColumn).IsEqualTo(current.Id)
                .ExecuteAsCollection<XJobTitleCollection>();

            StringBuilder sb = new StringBuilder();
            foreach (XJobTitle job in jobs)
            {
                sbValue.AppendFormat("{0},", job.Id);
                sbText.AppendFormat("{0}，", job.Name);
            }

            string roleValues = sbValue.ToString().TrimEnd(',');
            string roleTexts = sbText.ToString().TrimEnd('，');

            tbSelectedJobTitle.Text = roleTexts;
            hfSelectedJobTitle.Text = roleValues;

            // 打开编辑角色的窗口
            string selectJobTitleURL = String.Format("./user_select_jobtitle.aspx?ids=<script>{0}</script>", hfSelectedJobTitle.GetValueReference());
            tbSelectedJobTitle.OnClientTriggerClick = Window1.GetSaveStateReference(hfSelectedJobTitle.ClientID, tbSelectedJobTitle.ClientID)
                    + Window1.GetShowReference(selectJobTitleURL, "选择用户拥有的职称");

        }
        #endregion

        #region old code

        //private void BindDDLDept(XUser current)
        //{
        //    XDeptCollection items = new Select().From<XDept>()
        //        .OrderAsc(XDept.SortIndexColumn.ColumnName)
        //        .ExecuteAsCollection<XDeptCollection>();
        //    List<CustomDept> mys = CustomDeptHelper.GetMyDeptCollection(items);

        //    // 绑定到下拉列表（启用模拟树功能和不可选择项功能）
        //    ddlDept.EnableSimulateTree = true;
        //    ddlDept.DataTextField = "Name";
        //    ddlDept.DataValueField = "Id";
        //    ddlDept.DataSimulateTreeLevelField = "TreeLevel";
        //    ddlDept.DataEnableSelectField = "Enabled";
        //    ddlDept.DataSource = mys;
        //    ddlDept.DataBind();

        //    ddlDept.Items.Insert(0, new FineUI.ListItem("--请选择--", "-1"));
        //    //ddlDept.SelectedValue = current.DeptId == null ? "-1" : current.DeptId.ToString();
        //} 
        #endregion
        #endregion

        #region Events

        protected void btnSaveClose_Click(object sender, EventArgs e)
        {
            using (TransactionScope scope = new TransactionScope())
            {
                int userId = GetQueryIntValue("id");
                XUser item = XUser.FetchByID(userId);
                item.Name = tbxName.Text.Trim();
                item.ChineseName = tbxRealName.Text.Trim();
                item.Gender = ddlGender.SelectedValue;
                item.CompanyEmail = tbxEmail.Text.Trim();
                item.PersonalEmail = tbxPersonalEmail.Text.Trim();
                item.CellPhone = tbxCellPhone.Text.Trim();
                item.OfficePhone = tbxOfficePhone.Text.Trim();
                item.OfficePhoneExt = tbxOfficePhoneExt.Text.Trim();
                item.HomePhone = tbxHomePhone.Text.Trim();
                item.Remark = tbxRemark.Text.Trim();
                item.Enabled = cbxEnabled.Checked;
                //item.RoleId = Convert.ToInt32(ddlRole.SelectedValue);
                //item.DeptId = Convert.ToInt32(ddlDept.SelectedValue);
                item.Save();


                // 删除用户所属的所有角色
                new Delete().From(XRoleUser.Schema)
                    .Where(XRoleUser.UserIdColumn).IsEqualTo(userId)
                    .Execute();

                // 添加所有角色
                if (!String.IsNullOrEmpty(hfSelectedRole.Text))
                {
                    XRoleUserCollection roleUsers = new XRoleUserCollection();
                    foreach (string roleIdStr in hfSelectedRole.Text.Split(','))
                    {
                        int id = Convert.ToInt32(roleIdStr);

                        XRoleUser roleUser = new XRoleUser();
                        roleUser.RoleId = id;
                        roleUser.UserId = userId;

                        roleUsers.Add(roleUser);
                    }
                    roleUsers.SaveAll();
                }


                // 删除用户所属的所有部门
                new Delete().From(XDeptUser.Schema)
                    .Where(XDeptUser.UserIdColumn).IsEqualTo(userId)
                    .Execute();

                // 添加所有部门
                if (!String.IsNullOrEmpty(hfSelectedDept.Text))
                {
                    XDeptUser deptUser = new XDeptUser();
                    deptUser.DeptId = Convert.ToInt32(hfSelectedDept.Text);
                    deptUser.UserId = userId;
                    deptUser.Save();
                }



                // 删除用户所属的所有职称
                new Delete().From(XJobTitleUser.Schema)
                    .Where(XJobTitleUser.UserIdColumn).IsEqualTo(userId)
                    .Execute();

                // 添加所有职称
                if (!String.IsNullOrEmpty(hfSelectedJobTitle.Text))
                {
                    XJobTitleUserCollection jobtitleUsers = new XJobTitleUserCollection();
                    foreach (string idStr in hfSelectedJobTitle.Text.Split(','))
                    {
                        int id = Convert.ToInt32(idStr);

                        XJobTitleUser jobtitleUser = new XJobTitleUser();
                        jobtitleUser.JobTitleId = id;
                        jobtitleUser.UserId = userId;

                        jobtitleUsers.Add(jobtitleUser);
                    }
                    jobtitleUsers.SaveAll();
                }


                scope.Complete();
            }

            //Alert.Show("保存成功！", String.Empty, Alert.DefaultIcon, ActiveWindow.GetHidePostBackReference());
            PageContext.RegisterStartupScript(ActiveWindow.GetHidePostBackReference());
        }

        #endregion

    }
}
