﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using SubSonic;
using FineUI;
using System.Text;

namespace AppBox.admin
{
    public partial class user_view : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Module

        /// <summary>
        /// 当前页面所属的模块
        /// </summary>
        public override ModuleType Module
        {
            get
            {
                return ModuleType.CoreUser;
            }
        }

        #endregion

        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            btnClose.OnClientClick = ActiveWindow.GetHideReference();

            int id = GetQueryIntValue("id");
            XUser current = XUser.FetchByID(id);
            if (current == null)
            {
                // 参数错误，首先弹出Alert对话框然后关闭弹出窗口
                Alert.Show("参数错误！", String.Empty, ActiveWindow.GetHideReference());
                return;
            }


            labName.Text = current.Name;
            labRealName.Text = current.ChineseName;
            labEmail.Text = current.CompanyEmail;
            labPersonalEmail.Text = current.PersonalEmail;
            labCellPhone.Text = current.CellPhone;
            labOfficePhone.Text = current.OfficePhone;
            labOfficePhoneExt.Text = current.OfficePhoneExt;
            labHomePhone.Text = current.HomePhone;
            labRemark.Text = current.Remark;
            labEnabled.Text = current.Enabled ? "启用" : "禁用";
            labGender.Text = current.Gender;

            //if (current.RoleId != null && current.RoleId != -1)
            //{
            //    XRole role = new Select().From<XRole>()
            //        .Where(XRole.IdColumn).IsEqualTo(current.RoleId)
            //        .ExecuteSingle<XRole>();

            //    labRole.Text = role.Name;
            //}

            // 表关联查询用户所属的角色列表
            XRoleCollection roles = new Select().From(XRole.Schema)
                .InnerJoin(XRoleUser.RoleIdColumn, XRole.IdColumn)
                .Where(XRoleUser.UserIdColumn).IsEqualTo(current.Id)
                .ExecuteAsCollection<XRoleCollection>();

            StringBuilder sb = new StringBuilder();
            foreach (XRole role in roles)
            {
                sb.AppendFormat("{0}，", role.Name);
            }
            labRole.Text = sb.ToString().TrimEnd('，');

            /*
            if (current.DeptId != null && current.DeptId != -1)
            {
                XDept dept = new Select().From<XDept>()
                    .Where(XDept.IdColumn).IsEqualTo(current.DeptId)
                    .ExecuteSingle<XDept>();

                labDept.Text = dept.Name;
            }
             * */
            // 拥有职称
            

            // 初始化角色复选框列表的选择项
            XJobTitleCollection jobs = new Select().From(XJobTitle.Schema)
                .InnerJoin(XJobTitleUser.JobTitleIdColumn, XJobTitle.IdColumn)
                .Where(XJobTitleUser.UserIdColumn).IsEqualTo(current.Id)
                .ExecuteAsCollection<XJobTitleCollection>();

            sb = new StringBuilder();
            foreach (XJobTitle job in jobs)
            {
                sb.AppendFormat("{0}，", job.Name);
            }

            labJobTitle.Text = sb.ToString().TrimEnd('，');

            // 所属部门
            // 初始化角色复选框列表的选择项
            XDeptCollection depts = new Select().From(XDept.Schema)
                .InnerJoin(XDeptUser.DeptIdColumn, XDept.IdColumn)
                .Where(XDeptUser.UserIdColumn).IsEqualTo(current.Id)
                .ExecuteAsCollection<XDeptCollection>();

            if (depts.Count > 0)
            {
                labDept.Text = depts[0].Name;
            }
        }

        #endregion

    }
}
