﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Security;
using SubSonic;
using FineUI;
using System.Security.Principal;
using System.Text;


namespace AppBox
{
    public partial class _default : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            //string[] loginImages = new string[] { "login_1.png", "login_2.png", "login_3.png", "login_4.png", "login_6.png" };
            //imageLogin.ImageUrl = "~/res/images/login/" + loginImages[new Random().Next(0, loginImages.Length)];

            // 如果用户已经登录，则重定向到管理首页
            if (User.Identity.IsAuthenticated)
            {
                Response.Redirect(FormsAuthentication.DefaultUrl);
            }

            Window1.Title = String.Format("系统登录（AppBox v{0}）", GetProductVersion());

        }

        #endregion

        #region Events

        protected void btnSubmit_Click(object sender, EventArgs e)
        {
            string userName = tbxUserName.Text.Trim();
            string password = tbxPassword.Text.Trim();

            XUser user = new Select().From<XUser>()
                .Where(XUser.NameColumn).IsEqualTo(userName)
                .ExecuteSingle<XUser>();

            if (user != null)
            {
                if (PasswordUtil.ComparePasswords(user.Password, password))
                {
                    if (!user.Enabled)
                    {
                        Alert.Show("用户未启用，请联系管理员！");
                    }
                    else
                    {
                        // 登录成功
                        logger.Info(String.Format("登录成功：用户“{0}”", user.Name));

                        LoginSuccess(user);

                        //LoginOnlineUser(user);
                        //FormsAuthentication.RedirectFromLoginPage(userName, true);
                        return;
                    }
                }
                else
                {
                    logger.Warn(String.Format("登录失败：用户“{0}”密码错误", userName));
                    Alert.Show("用户名或密码错误！");
                    return;
                }

            }
            else
            {
                logger.Warn(String.Format("登录失败：用户“{0}”不存在", userName));
                Alert.Show("用户名或密码错误！");
                return;
            }
        }


        private void LoginSuccess(XUser user)
        {
            LoginOnlineUser(user);

            // 用户所属的角色，以逗号分隔
            StringBuilder sb = new StringBuilder();
            XRoleUserCollection roleUsers = new Select().From(XRoleUser.Schema)
                .Where(XRoleUser.UserIdColumn).IsEqualTo(user.Id)
                .ExecuteAsCollection<XRoleUserCollection>();
            foreach (XRoleUser roleUser in roleUsers)
            {
                sb.AppendFormat("{0},", roleUser.RoleId);
            }
            string roleIds = sb.ToString().TrimEnd(',');


            FormsAuthenticationTicket ticket = new FormsAuthenticationTicket(1, 
                user.Name, 
                DateTime.Now,
                DateTime.Now.AddMinutes(120), 
                true,
                roleIds, 
                FormsAuthentication.FormsCookiePath);
            string hashTicket = FormsAuthentication.Encrypt(ticket);
            HttpCookie userCookie = new HttpCookie(FormsAuthentication.FormsCookieName, hashTicket);
            userCookie.HttpOnly = true;
            userCookie.Expires = DateTime.Now.AddMinutes(120);
            Response.Cookies.Add(userCookie);
            Response.Redirect(FormsAuthentication.DefaultUrl);
        }


        #endregion
    }
}
