﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Security;
using SubSonic;
using Newtonsoft.Json.Linq;
using FineUI;


namespace AppBox
{
    public partial class main : PageBase
    {
        private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Page_Init

        protected void Page_Init(object sender, EventArgs e)
        {
            // 工具栏上的帮助菜单
            //Nii.JSON.JSONArray ja = new Nii.JSON.JSONArray(XConfigHelper.HelpList);
            JArray ja = JArray.Parse(XConfigHelper.HelpList);
            foreach (JObject jo in ja)
            {
                MenuButton menuItem = new MenuButton();
                menuItem.EnablePostBack = false;
                menuItem.Text = jo.Value<string>("Text");
                //menuItem.Icon = (Icon)StringUtil.EnumFromName(typeof(Icon), jo.Value<string>("Icon"));
                menuItem.Icon = IconHelper.String2Icon(jo.Value<string>("Icon"), true);
                menuItem.OnClientClick = String.Format("addExampleTab('{0}', '{1}', '{2}')", jo.Value<string>("ID"), ResolveUrl(jo.Value<string>("URL")), jo.Value<string>("Text"));

                btnHelp.Menu.Items.Add(menuItem);
            }

            // 用户可见的菜单列表
            List<CustomMenu> menus = ResolveUserMenuList();
            if (menus.Count == 0)
            {
                Response.Write("系统管理员尚未给你配置菜单！");
                Response.End();

                return;
            }


            // 注册客户端脚本，服务器端控件ID和客户端ID的映射关系
            JObject ids = GetClientIDS(regionPanel, regionTop, btnShowHideHeader, mainTabStrip, txtUser,
                txtOnlineUserCount, txtCurrentTime, btnRefresh);
            ids.Add("userName", User.Identity.Name);
            ids.Add("userIP", Request.UserHostAddress);
            ids.Add("onlineUserCount", GetOnlineCount());

            if (XConfigHelper.MenuType == "accordion")
            {
                Accordion accordionMenu = InitAccordionMenu(menus);
                ids.Add("treeMenu", accordionMenu.ClientID);
                ids.Add("menuType", "accordion");

            }
            else
            {
                Tree treeMenu = InitTreeMenu(menus);
                ids.Add("treeMenu", treeMenu.ClientID);
                ids.Add("menuType", "menu");

            }

            string idsScriptStr = String.Format("window.DATA={0};", ids.ToString(Newtonsoft.Json.Formatting.None));
            PageContext.RegisterStartupScript(idsScriptStr);
        }

        private JObject GetClientIDS(params ControlBase[] ctrls)
        {
            JObject jo = new JObject();
            foreach (ControlBase ctrl in ctrls)
            {
                jo.Add(ctrl.ID, ctrl.ClientID);
            }

            return jo;
        }

        #endregion

        #region InitAccordionMenu

        /// <summary>
        /// 创建手风琴菜单
        /// </summary>
        /// <param name="menus"></param>
        /// <returns></returns>
        private Accordion InitAccordionMenu(List<CustomMenu> menus)
        {
            Accordion accordionMenu = new Accordion();
            accordionMenu.ID = "accordionMenu";
            accordionMenu.EnableFill = true;
            accordionMenu.ShowBorder = false;
            accordionMenu.ShowHeader = false;
            regionLeft.Items.Add(accordionMenu);


            foreach (CustomMenu menu in menus)
            {
                if (menu.ParentId == 0)
                {
                    AccordionPane accordionPane = new AccordionPane();
                    accordionPane.Title = menu.Title;
                    accordionPane.Layout = Layout.Fit;
                    accordionPane.ShowBorder = false;
                    accordionPane.BodyPadding = "2px 0 0 0";
                    accordionMenu.Items.Add(accordionPane);

                    Tree innerTree = new Tree();
                    innerTree.EnableArrows = true;
                    innerTree.ShowBorder = false;
                    innerTree.ShowHeader = false;
                    innerTree.EnableIcons = false;
                    innerTree.AutoScroll = true;
                    accordionPane.Items.Add(innerTree);


                    // 生成树
                    ResolveMenuTree(menus, menu.Id, innerTree.Nodes);

                }
            }

            return accordionMenu;
        }

        #endregion

        #region InitTreeMenu

        /// <summary>
        /// 创建树菜单
        /// </summary>
        /// <param name="menus"></param>
        /// <returns></returns>
        private Tree InitTreeMenu(List<CustomMenu> menus)
        {
            Tree treeMenu = new Tree();
            treeMenu.ID = "treeMenu";
            treeMenu.EnableArrows = true;
            treeMenu.ShowBorder = false;
            treeMenu.ShowHeader = false;
            treeMenu.EnableIcons = false;
            treeMenu.AutoScroll = true;
            regionLeft.Items.Add(treeMenu);

            // 生成树
            ResolveMenuTree(menus, 0, treeMenu.Nodes);

            // 展开第一个树节点
            treeMenu.Nodes[0].Expanded = true;

            return treeMenu;
        }

        private List<CustomMenu> ResolveUserMenuList()
        {
            List<CustomMenu> menus = new List<CustomMenu>();
            // 全部的菜单列表
            List<CustomMenu> originalMenus = CustomMenuHelper.GetCustomMenus();

            // 当前用户所属角色可以显示的模块列表
            List<string> moduleNames = new List<string>();
            foreach (string roleId in GetIdentityRoleIDs().Split(','))
            {
                moduleNames.AddRange(RoleModulePowerHelper.GetRoleCanReadModuleList(Convert.ToInt32(roleId)));
            }

            // 当前用户所属角色可用的菜单列表
            foreach (CustomMenu menu in originalMenus)
            {
                // 如果此菜单不属于任何模块，或者此用户所属角色拥有对此模块的权限
                if (menu.Name == ModuleTypeHelper.Module2String(ModuleType.None) || moduleNames.Contains(menu.Name))
                {
                    menus.Add(menu);
                }
            }

            return menus;
        }

        /// <summary>
        /// 生成菜单树
        /// </summary>
        /// <param name="menus"></param>
        /// <param name="parentMenuId"></param>
        /// <param name="nodes"></param>
        private void ResolveMenuTree(List<CustomMenu> menus, int parentMenuId, FineUI.TreeNodeCollection nodes)
        {
            foreach (CustomMenu menu in menus)
            {
                if (menu.ParentId == parentMenuId)
                {
                    FineUI.TreeNode node = new FineUI.TreeNode();
                    nodes.Add(node);

                    node.Text = menu.Title;
                    node.IconUrl = menu.ImageUrl;
                    if (!String.IsNullOrEmpty(menu.NavigateUrl))
                    {
                        node.EnablePostBack = false;
                        node.NavigateUrl = ResolveUrl(menu.NavigateUrl);
                        //node.OnClientClick = String.Format("addTab('{0}','{1}','{2}')", node.NodeID, ResolveUrl(menu.NavigateUrl), node.Text.Replace("'", ""));
                    }

                    if (menu.IsTreeLeaf)
                    {
                        node.Leaf = true;
                    }
                    else
                    {
                        node.SingleClickExpand = true;
                    }

                    ResolveMenuTree(menus, menu.Id, node.Nodes);
                }
            }
        }

        #endregion


        #region Page_Load

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        private void LoadData()
        {
            //txtUser.Text = String.Format("欢迎您：<span style=\"font-weight:bold;color:red;\">{0}</span> [{1}]", User.Identity.Name, Request.UserHostAddress);
            //txtOnlineUserCount.Text = String.Format("在线用户：{0}", "2");

            //btnSysConfig.OnClientClick = Window1.GetShowReference("~/admin/config.aspx", "系统设置");

            litProductVersion.Text = String.Format("（AppBox v{0}）", GetProductVersion());
        }


        #endregion


        #region Events

        protected void btnExit_Click(object sender, EventArgs e)
        {
            FormsAuthentication.SignOut();
            FormsAuthentication.RedirectToLoginPage();
        }

        #endregion
    }
}
